package ch.zhaw.catan;

import java.util.*;

/**
 * New Class PLayer
 * This class is here to add players to the game.
 */
public class Player {

    private Config.Faction faction;
    private HashMap<Config.Resource,Integer> resources;
    private HashMap<Config.Structure,Integer> structureToUse;
    private int roadsToUse;
    private int settlementsToUse;
    private int citiesToUse;

    public Player (Config.Faction faction){
        //Datenfelder
        this.faction = faction;
        //Verfügbare Structures initialisieren
        structureToUse = new HashMap<>();
        structureToUse.put(Config.Structure.ROAD, Config.Structure.ROAD.getStockPerPlayer());
        structureToUse.put(Config.Structure.SETTLEMENT, Config.Structure.SETTLEMENT.getStockPerPlayer());
        structureToUse.put(Config.Structure.CITY, Config.Structure.CITY.getStockPerPlayer());
        //Ressourcen initialisiern
        resources = new HashMap<>();
        for(Config.Resource resource : Config.Resource.values()) {
            resources.put(resource,0);
        }
    }


    /**
     * This method returns all the resources the player has at the moment
     * @return HashMap with the count of every resource
     */
    public HashMap<Config.Resource,Integer> getResources() {
        return resources;
    }

    /**
     * This method returns player faction
     * @return
     */
    public Config.Faction getFaction() { return faction; }

    /**
     * This method returns for specific resource how much player possesess.
     * @param resource
     * @return
     */
    public int getSpecificResource(Config.Resource resource) { return resources.get(resource); }

    /**
     * This method adds a specific resource to resourcess
     * @param resource to add
     * @param numberToAdd how much to add
     */
    public void addResource(Config.Resource resource, int numberToAdd) {
        resources.put(resource, resources.get(resource) + numberToAdd);
    }

    /**
     * This method substracts a specific resource from resourcess but check first if player has enough resources.
     * @param resource to substract
     * @param numberToSubstract how much to substract
     * @return true if resource has been substracted false if player has not enough resources
     */
    public boolean substractResource(Config.Resource resource, int numberToSubstract) {
        int inPossesion = resources.get(resource);
        if(inPossesion - numberToSubstract < 0) {
            return false;
        }
        resources.put(resource,inPossesion - numberToSubstract);
        return true;
    }

    //todo java doc aktualisieren
    /**
     * This method has to be used when a player wants to build a settlement. It checks if a player has enough roads
     * and resources to build one more. If the player is able to build, this method subtracts the buildcost from the resources
     * in possession by the player.
     * @return true if road can be created false if road can't be created.
     */
    public boolean build(Config.Structure structure){
        List<Config.Resource> costs = structure.getCosts();
        if(structureToUse.get(structure) < 1 || !checkResourceToBuild(costs)){
            return false;
        }
        structureToUse.put(structure, structureToUse.get(structure) - 1);
        return true;
    }



   //returns true if player has enough resources else false
    private boolean checkResourceToBuild(List<Config.Resource> list) {
        HashMap<Config.Resource, Integer> costs = new HashMap<>();
        for(Config.Resource resource : list){
            if(costs.containsKey(resource)){
                costs.put(resource, costs.get(resource) + 1);
            } else {
                costs.put(resource, 1);
            }
        }
        for (Config.Resource resource : costs.keySet()) {
            if (resources.get(resource) < costs.get(resource)) {
                return false;
            }
        }
        return true;
    }


}
